@@ -6,8 +6,6 @@ import json
 import logging
 import random
 import re
-import signal
-import socket
 import time as mod_time
 import uuid
 
@@ -17,11 +15,12 @@ import vcode as mod_vcode
 from CodeConvert import CodeConvert as cc
 from redis import StrictRedis
 from redis._compat import iteritems, xrange
+from redis.client import bool_ok
 from redis.exceptions import ResponseError, WatchError
+from redis_extensions.expires import BaseRedisExpires
 from TimeConvert import TimeConvert as tc
 
 from .compat import basestring, bytes
-from .expires import BaseRedisExpires
 
 
 logger = logging.getLogger('redis_extensions')
@@ -33,33 +32,6 @@ KEY_PREFIX = 'r:'  # Prefix of redis-extensions used key
 WARNING_LOG = '``{0}`` used, may be very very very slow when keys\' amount very large'  # ``r.keys()`` and ``r.scan_iter()`` not support use
 
 
-POLL_QUEUE_CONTINUE_FLAG = True
-
-
-# Signal Handler
-def sigintHandler(signum, frame):
-    global POLL_QUEUE_CONTINUE_FLAG
-    POLL_QUEUE_CONTINUE_FLAG = False
-
-
-# signal.SIGKILL, `KILL -9`, unblockable
-for signum in [signal.SIGHUP, signal.SIGINT, signal.SIGTERM, signal.SIGTSTP]:
-    signal.signal(signum, sigintHandler)
-
-
-# Get the local ip
-def get_network_ip():
-    try:
-        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
-        s.setsockopt(socket.SOL_SOCKET, socket.SO_BROADCAST, 1)
-        s.connect(('<broadcast>', 0))
-        ip = s.getsockname()[0]
-        s.close()
-    except OSError:
-        ip = '127.0.0.1'
-    return ip
-
-
 class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
     """
     Extension of [redis-py](https://github.com/andymccurdy/redis-py)'s StrictRedis.
@@ -75,8 +47,6 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
         super(StrictRedisExtensions, self).__init__(*args, **kwargs)
 
     def __str(self, x):
-        if isinstance(x, int):
-            return str(x)
         return x if isinstance(x, basestring) else bytes(x)
 
     def __local_ymd(self, format='%Y-%m-%d'):
@@ -196,12 +166,19 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
 
     # # Keys Section(Rename Relative)
     def quiet_rename(self, src, dst):
-        if self.exists(src):
-            try:
-                return self.rename(src, dst)
-            except ResponseError:
-                pass
-        return False
+        # if self.exists(src):
+        #     try:
+        #         return self.rename(src, dst)
+        #     except ResponseError:
+        #         pass
+        # return False
+        quiet_rename_script = """
+        if redis.call('exists', KEYS[1]) == 1 then
+            return redis.call('rename', KEYS[1], KEYS[2])
+        else
+            return ''
+        end"""
+        return bool_ok(self.eval(quiet_rename_script, 2, src, dst))
 
     # Strings Section
     def get_delete(self, name):
@@ -330,7 +307,7 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
         """
         if isinstance(time, datetime.timedelta):
             time = time.seconds + time.days * 24 * 3600
-        return self.zadd(name, mod_time.time() + time, value)
+        return self.zadd(name, {value: mod_time.time() + time})
 
     def lrange_ex(self, name):
         cur_time = mod_time.time()
@@ -373,8 +350,7 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
         # Srandmember isn't random enough
         # When number is close to ``the number of elements inside the set``
         memebers = self.srandmember(name, number=number)
-        if memebers:
-            random.shuffle(memebers)
+        random.shuffle(memebers)
         return memebers
 
     # ZSorts(Sorted Sets) Section
@@ -519,14 +495,16 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
 
     def zaddwithstamps(self, name, *args, **kwargs):
         desc = 'desc' in kwargs and kwargs.pop('desc')
-        pieces = [item if index % 2 else self.__stampscore(item, desc) for index, item in enumerate(args)]
-        for pair in iteritems(kwargs):
-            pieces.append(self.__stampscore(pair[1], desc))
-            pieces.append(pair[0])
-        return self.zadd(name, *pieces)
+        mapping = kwargs.pop('mapping') if 'mapping' in kwargs else {}
+        for idx, item in enumerate(args):
+            if idx % 2:
+                mapping[item] = self.__stampscore(args[idx + 1], desc)
+        for k, v in kwargs.items():
+            mapping[k] = self.__stampscore(v, desc)
+        return self.zadd(name, mapping)
 
     def zincrbywithstamps(self, name, value, amount=1, desc=False):
-        return self.zadd(name, self.__stampscore(self.rawscore(self.zscore(name, value)) + amount, desc), value)
+        return self.zadd(name, {value: self.__stampscore(self.rawscore(self.zscore(name, value)) + amount, desc)})
 
     def zrawscore(self, name, value):
         """
@@ -726,13 +704,7 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
                 pass
         return False
 
-    def delete_lock(self, name):
-        """
-        Delete lock for ``name``.
-        """
-        return self.delete(self.__lock_key(name))
-
-    def exists_lock(self, name, regex=False):
+    def lock_exists(self, name, regex=False):
         """
         Check lock for ``name`` exists or not.
         """
@@ -1052,13 +1024,10 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
         return '{0}queue:{1}'.format(KEY_PREFIX, queue)
 
     def execute_later(self, queue, name, args=None, delayed=KEY_PREFIX + 'delayed:default', delay=0, short_uuid=False, enable_queue=False):
-        """
-        Producer of delay execute.
-        """
         identifier = self.__uuid(short_uuid)
         item = json.dumps([identifier, queue, name, args])
         if delay > 0:
-            self.zadd(delayed, mod_time.time() + delay, item)
+            self.zadd(delayed, {item: mod_time.time() + delay})
         else:
             if enable_queue:
                 self.rpush(self.__queue_key(queue), item)
@@ -1075,121 +1044,48 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
             logger.error(e)
             return None
 
-    def release_poll_queue_lock(self, delayed, final_logger=None):
-        if not delayed:
-            return
-        item = self.zrange(delayed, 0, 0, withscores=True)
-        if not item:
-            return
-        item = item[0][0]
-        identifier, queue, name, args = json.loads(item)
-        final_logger.info('  * Release lock: {0}'.format(identifier))
-        return self.delete_lock(identifier)
-
-    def __release_lock_when_launch(self, release_lock_when_launch, release_lock_eth0_inet_addr, final_process_lock_key, delayed, final_logger):
-        if not release_lock_when_launch:
-            return
-        if not release_lock_eth0_inet_addr:
-            final_logger.info('>>> Release lock should pass `release_lock_eth0_inet_addr`')
-            return
-        eth0_inet_addr = get_network_ip()
-        if eth0_inet_addr not in release_lock_eth0_inet_addr:
-            final_logger.info('>>> Release lock `release_lock_eth0_inet_addr` not match')
-            return
-        final_logger.info('>>> Release process lock start')
-        self.delete_lock(final_process_lock_key)
-        final_logger.info('>>> Release process lock end')
-        final_logger.info('>>> Release item lock start')
-        self.release_poll_queue_lock(delayed, final_logger=final_logger)
-        final_logger.info('>>> Release item lock end')
-
-    def poll_queue(self, callbacks={}, delayed=KEY_PREFIX + 'delayed:default', enable_auto_zrem=False, enable_queue=False, enable_process_lock=False, process_lock_key=None, release_lock_when_launch=True, release_lock_eth0_inet_addr=None, release_lock_when_error=True, delayed_logger=None, unlocked_warning_func=None):
-        """
-        Consumer of delay execute.
-
-        ``unlocked_warning_func`` indicates callback function when ``acquire_lock`` fail.
-
-        ``enable_auto_zrem`` indicates whether enable auto zrem or not. ``True`` for at most once, ``False`` for at least once.
-
-        ``enable_queue`` indicates whether enable queue or not.
-
-        ``enable_process_lock`` indicates whether enable process lock or not.
-
-        ``process_lock_key`` indicates acquire process lock key.
-
-        ``release_lock_when_launch`` indicates whether release lock when launch or not. This is for ``restart``.
-
-        ``release_lock_eth0_inet_addr`` indicates eth0 inet addr, which can release lock when ``release_lock_when_launch``.
-
-        ``release_lock_key`` indicates acquire lock key when ``release_lock_when_launch``.
-
-        ``release_lock_key_expire`` indicates expire time of ``release_lock_key``.
-
-        ``release_lock_when_error`` indicates whether release lock when error or not.
-        """
+    def poll_queue(self, callbacks={}, delayed=KEY_PREFIX + 'delayed:default', unlocked_warning_func=None, enable_queue=False, release_lock_when_error=True):
         callbacks = {k: self.__callable_func(v) for k, v in iteritems(callbacks)}
         callbacks = {k: v for k, v in iteritems(callbacks) if v}
 
-        final_process_lock_key = 'process:lock:{0}'.format(process_lock_key or delayed)
-        final_logger = delayed_logger or logger
-
-        final_logger.info('>>> Available callbacks ({0}):'.format(len(callbacks)))
+        logger.info('Available callbacks ({0}):'.format(len(callbacks)))
         for k, v in iteritems(callbacks):
-            final_logger.info('  * {0}: {1}'.format(k, v))
-
-        self.__release_lock_when_launch(release_lock_when_launch, release_lock_eth0_inet_addr, final_process_lock_key, delayed, final_logger)
-
-        process_lock = None
-        while POLL_QUEUE_CONTINUE_FLAG:
-            if enable_process_lock:
-                # Release process lock
-                if process_lock:
-                    self.release_lock(final_process_lock_key, process_lock)
-
-                # Acquire process lock
-                process_lock = self.acquire_lock(final_process_lock_key)
-                if not process_lock:
-                    continue
+            logger.info('* {0}: {1}'.format(k, v))
 
+        while True:
             item = self.zrange(delayed, 0, 0, withscores=True)
 
             if not item or item[0][1] > mod_time.time():
                 mod_time.sleep(.01)
                 continue
 
-            final_logger.info(item)
+            logger.info(item)
 
             item = item[0][0]
             identifier, queue, name, args = json.loads(item)
 
-            item_lock = self.acquire_lock(identifier)
-            if not item_lock:
-                # Call ``unlocked_warning_func`` if exists when ``acquire_lock`` fail
+            locked = self.acquire_lock(identifier)
+            if not locked:
+                # Call ``unlocked_warning_func`` if exists when ``acquire_lock`` fails
                 if unlocked_warning_func:
                     unlocked_warning_func(queue, name, args)
                 continue
 
-            # At most once 最多消费一次
-            if enable_auto_zrem and self.zrem(delayed, item):
-                if enable_queue:
-                    self.rpush(self.__queue_key(queue), item)
-                self.release_lock(identifier, item_lock)
-
             # Callbacks
             if queue in callbacks:
                 try:
                     callbacks[queue](name, args)
                 except Exception as e:
-                    final_logger.error(e)
-                    if not release_lock_when_error:
-                        continue
+                    logger.error(e)
+                    if release_lock_when_error:
+                        self.release_lock(identifier, locked)
+                    continue
 
-            # At least once 最少消费一次
-            if not enable_auto_zrem and self.zrem(delayed, item):
+            if self.zrem(delayed, item):
                 if enable_queue:
                     self.rpush(self.__queue_key(queue), item)
 
-            self.release_lock(identifier, item_lock)
+            self.release_lock(identifier, locked)
 
     # HotKey Section
     def hotkey(self, gfunc=None, gargs=None, gkwargs=None, sfunc=None, sargs=None, skwargs=None, update_timeout=1000, short_uuid=False):
@@ -1301,7 +1197,6 @@ class StrictRedisExtensions(BaseRedisExpires, StrictRedis):
     zgte = zge
     zlte = zle
     vcode_status = vcode_exists
-    lock_exists = exists_lock
 
     # Delete => Del
     delkeys = del_keys = delete_keys
